// Definicija klase polinoma (Poli).

#include <iostream>
using namespace std;

class Poli {
  int n; double* a;          // Red polinoma i pokazivac na koeficijente.
  typedef const Poli CP;     // Nepromenljiv polinom.
  void kopiraj(const double* a, int n);           // Kopiranje u polinom.
  void premesti(Poli& p)                          // Premestanje u polinom.
    { n = p.n; a = p.a; p.a = nullptr; }
  void brisi()                                    // Oslobadjanje memorije.
    { delete[] a; a = nullptr; n = -1; }
public:                                           // Inicijalizacija:
  Poli() { a = nullptr; n = -1; }                 //   - prazan polinom
  Poli(const double* a, int n) { kopiraj(a, n); } //   - nizom
  Poli(CP& p) { kopiraj(p.a, p.n); }              //   - polinomom
  Poli(Poli&& p) { premesti(p); }                 //   - polinomom
  ~Poli() { brisi(); }                            // Destruktor.
  Poli& operator=(CP& p) {                        // Dodela vrednosti
    if (this != &p) { brisi(); kopiraj(p.a, p.n); }  // kopiranjem.
    return *this;
  }
  Poli& operator=(Poli&& p) {                     // Dodela vrednosti
    if (this != &p) { brisi(); premesti(p); }        // premestanjem.
    return *this;
  }
  void niz(double* a, int& n) const;         // Smestanje polinoma u niz.
  int red() const { return n; }              // Red polinoma.
  double operator()(double x) const;         // Vrendost polinoma.
  double& operator[](int ind);               // Dohvatanje koeficijenta.
  const double& operator[](int ind) const;
  friend Poli operator+(CP& p1, CP& p2);                  // p1 +  p2
  friend Poli operator-(CP& p1, CP& p2);                  // p1 -  p2
  friend Poli operator*(CP& p1, CP& p2);                  // p1 *  p2
  Poli& operator+=(CP& p2) { return *this = *this + p2; } // p1 += p2
  Poli& operator-=(CP& p2) { return *this = *this - p2; } // p1 -= p2
  Poli& operator*=(CP& p2) { return *this = *this * p2; } // p1 *= p2
  friend istream& operator>>(istream& ut, Poli& p); // Citanje polinoma
  friend ostream& operator<<(ostream& it, CP& p);   // Pisanje polinoma
private:
  enum Greska { G_RED, G_IND };                     // Sifre gresaka.
  [[noreturn]] static void greska (Greska g);       // Obrada greske.
};

